<?php

session_start();
ini_set('display_errors', 1);

require $_SERVER['DOCUMENT_ROOT'] . "/idealpatios2/contact/Mailer.php";

$config = [
    // The URL of the form e.g. /contact.html
    'FORM_URL' => '/contact.html',
    'RECAPTCHA_SECRET' => "6Ld2wD8qAAAAAALgWCmjS6AMcobdRbn8p8_dTPxO",
    // higher the number the more likely it's a genuine request, change this value depending on how many false positives or spam gets through
    'RECAPTCHA_SCORE_THRESHOLD' => 0.5,

    'THANKS_URL' => '/idealpatios2/thanks.html',
    // Look for URLs in the inputs and block if found
    'BLOCK_URLS' => true,
    // Should use the global blacklist
    'USE_REMOTE_BLACKLIST' => true,
    // If above remote list is unsuitable in this instance, you can define a custom list here or leave empty
    'CUSTOM_BLACKLIST' => [
        'Company: Google',
    ],

    "MAIL" => [
        "TO_ADDRESSES" => ["derek.mcarthur@nxds.com"],
        "BCC_ADDRESSES" => [],
        "FROM_ADDRESS" => "info@ideal-exteriors.co.uk",
        "SUBJECT" => "New contact enquiry received",
    ],

    "ERRORS" => [
        "URL_FOUND" => "You cannot add any links in this form.",
        "RECAPTCHA_FAILED" => "Robot verification failed, please try again.",
        "BLACKLISTED_TERM_FOUND" => "Could not submit form",
        "INVALID_EMAIL" => "Not a valid email",
    ],
];
// Code inside this IF statement will only get called once form is submitted
if ($_SERVER['REQUEST_METHOD'] == 'POST')
{
    if (! validate_recaptcha()) {
        redirect_back_to_form($config['ERRORS']['RECAPTCHA_FAILED']);
    }

    // Loop through POST params and sanitise the data for HTML tags, etc
    $sanitisedInputs = [];
    foreach($_POST as $key=>$input) {
        
        if ($key !== 'photos') {
            $sanitisedInputs[$key] = sanitise_input($input);
        }
    }

    // Loop through the sanitised data and validate the inputs
    $validatedInputs = [];

    if (isset($_FILES['photos'])) {
        $files = $_FILES['photos'];

        // Check if more than 4 files were uploaded
        if (count($files['name']) > 4) {
            $errors[] = 'You can only upload a maximum of 4 images.';
        } else {
            for ($i = 0; $i < count($files['name']); $i++) {

                // If no errors, move the uploaded file to the target directory
                if (empty($errors)) {
                    $validatedInputs['photos'][] = $files['tmp_name'][$i];
                }
            }
        }
    }

    foreach($sanitisedInputs as $key=>$input) {
        $validatedInputs[$key] = validate_input($input, $key);
    }

    // Take this data and check against list of blacklist terms
    check_inputs_for_blacklist_terms($validatedInputs);

    sendEmail($validatedInputs);

    header("Location: " . $config['THANKS_URL']);
}

/**
 * Helper function to sanitise user input data
 * @param mixed $data input value to be sanitised
 * @return string sanitised data
 */
function sanitise_input($data)
{
    global $config;

    $value = trim($data);
    $value = stripslashes($value);
    $value = htmlspecialchars($value);

    return $value;
}

/**
 *
 * @param mixed $data
 * @param mixed $dataType
 * @return void
 */
function validate_input($data, $key)
{
    global $config;

    // Validate against URLs if config setting enabled and not looking in an email related field
    if ($config['BLOCK_URLS']) {

        $foundURL = str_contains($data, "http") || str_contains($data, 'www.');
        if ($foundURL) {
            redirect_back_to_form($config['ERRORS']['URL_FOUND']);
        }
    }

    if ($key == 'email') {
        if (!filter_var($data, FILTER_VALIDATE_EMAIL)) {
            redirect_back_to_form($config['ERRORS']['INVALID_EMAIL']);
        }
    }

    return $data;
}

/**
 *
 * @param mixed $validatedInputs user entered values after sanitisation and validation
 * @return void
 */
function check_inputs_for_blacklist_terms($validatedInputs)
{
    global $config;

    $blackListArray = [];

    if ($config['USE_REMOTE_BLACKLIST']) {
        $blackListArray = json_decode(file_get_contents("https://new.nexusspace.co.uk/contactForm/blacklist.json"));
    } else {
        $blackListArray = $config['CUSTOM_BLACKLIST'];
    }

    foreach($validatedInputs as $key=>$input) {
        if ($key === 'photos') continue;
        foreach ($blackListArray as $blacklistTerm) {
            if (str_contains($input, $blacklistTerm)) {
                redirect_back_to_form($config['ERRORS']['BLACKLISTED_TERM_FOUND']);
            }
        }
    }
}

/**
 * Validate form submission against Google reCAPTCHA v3 and compare score to config threshold
 * @return bool true if passed
 */
function validate_recaptcha()
{
    global $config;

    if (!empty($config['RECAPTCHA_SECRET']) && isset($_POST['g-recaptcha-response'])) {
        $recaptcha_response = $_POST['g-recaptcha-response'];

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, 'https://www.google.com/recaptcha/api/siteverify');
        // Receive response from server
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        // POST request
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt(
            $ch,
            CURLOPT_POSTFIELDS,
            http_build_query([
                'secret' => $config['RECAPTCHA_SECRET'],
                'response' => $recaptcha_response,
            ])
        );

        $response = curl_exec($ch);
        curl_close($ch);


        $resp = json_decode($response);
        return $resp->success && $resp->score > $config['RECAPTCHA_SCORE_THRESHOLD'];
    }

    return false;
}


function sendEmail($validatedInputs)
{
    global $config;

    $mail = new Mailer($config['MAIL']);

    $mail->setBody($validatedInputs);
    $mail->send();
}

/**
 * Helper function to redirect back to the form if bad submission
 * @param string $error custom error message to display on the form when redirected
 * @return void
 */
function redirect_back_to_form($error)
{
    global $config;

    if ($error) {
        $_SESSION['error'] = $error;
    }

    header("Location: ".$config['FORM_URL']);
    die();
}

?>
